@@ -9,7 +9,7 @@ selection method.
 """
 
 #
-# Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007 The SCons Foundation
+# Copyright (c) 2001 - 2014 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -31,7 +31,7 @@ selection method.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/home/scons/scons/branch.0/baseline/src/engine/SCons/Tool/aixc++.py 0.97.D001 2007/05/17 11:35:19 knight"
+__revision__ = "src/engine/SCons/Tool/aixc++.py  2014/09/27 12:51:43 garyo"
 
 import os.path
 
@@ -43,34 +43,33 @@ packages = ['vacpp.cmp.core', 'vacpp.cmp.batch', 'vacpp.cmp.C', 'ibmcxx.cmp']
 
 def get_xlc(env):
     xlc = env.get('CXX', 'xlC')
-    xlc_r = env.get('SHCXX', 'xlC_r')
-    return SCons.Platform.aix.get_xlc(env, xlc, xlc_r, packages)
-
-def smart_cxxflags(source, target, env, for_signature):
-    build_dir = env.GetBuildPath()
-    if build_dir:
-        return '-qtempinc=' + os.path.join(build_dir, 'tempinc')
-    return ''
+    return SCons.Platform.aix.get_xlc(env, xlc, packages)
 
 def generate(env):
     """Add Builders and construction variables for xlC / Visual Age
     suite to an Environment."""
-    path, _cxx, _shcxx, version = get_xlc(env)
-    if path:
+    path, _cxx, version = get_xlc(env)
+    if path and _cxx:
         _cxx = os.path.join(path, _cxx)
-        _shcxx = os.path.join(path, _shcxx)
+
+    if 'CXX' not in env:
+        env['CXX'] = _cxx
 
     cplusplus.generate(env)
 
-    env['CXX'] = _cxx
-    env['SHCXX'] = _shcxx
-    env['CXXVERSION'] = version
-    env['SHOBJSUFFIX'] = '.pic.o'
+    if version:
+        env['CXXVERSION'] = version
     
 def exists(env):
-    path, _cxx, _shcxx, version = get_xlc(env)
+    path, _cxx, version = get_xlc(env)
     if path and _cxx:
         xlc = os.path.join(path, _cxx)
         if os.path.exists(xlc):
             return xlc
     return None
+
+# Local Variables:
+# tab-width:4
+# indent-tabs-mode:nil
+# End:
+# vim: set expandtab tabstop=4 shiftwidth=4:
